# -*- coding: utf-8 -*-
"""
Created on Thu Apr 16 13:52:51 2020

@author: mi
"""



from numba import jit, njit, objmode, prange
from numba.typed import List

import numpy as np

@njit
def translation(ref1, ref2):
    s = np.zeros_like(ref1)    
    with objmode(s='intp[:,:]'):
        s = np.abs(np.fft.ifft2(np.fft.fft2(ref1) * np.conjugate(np.fft.fft2(ref2))))
    with objmode(arr='intp[:]'):
        arr = np.asarray(np.where(s == np.min(s))).ravel()
    return [ arr[0] - ref1.shape[0]//2, arr[1]- ref1.shape[0]//2]

@jit(nopython=True, parallel=True)
def translations(size:int, subs:np.ndarray, ref:np.ndarray):     
    offsets = List()
    x = 0
    y = 0
    i = 0 
    for i in prange(size):          
        x, y = translation(subs[i], ref)
        offsets.append((i,x,y))
    return offsets


# -*- coding: utf-8 -*-
"""
Created on Thu Jan 30 20:05:33 2020

@author: mi
"""

import numpy as np
import matplotlib.pyplot as plt
from skimage.filters import gaussian
from random import randint


class WFSGenerationError(Exception):
    pass

def drow_circle(r, offset = [0,0], sigma = 1):
    
    width, height = r*2, r*2
    img = np.zeros((width, height))
    X = np.arange(width).reshape(width,1)
    Y = np.arange(height).reshape(1,height)

    mask_2 = ((X - offset[0]- width//2) ** 2 + (Y  -offset[1]- height//2)**2) < (1)**2
    img[mask_2] = 1    
    img_smooth = gaussian(img, sigma)
    
   ## print(sum(img_smooth[0,:]+img_smooth[:,0]+img_smooth[-1,:]+img_smooth[:,-1]))
    if sum(img_smooth[0,:]+img_smooth[:,0]+img_smooth[-1,:]+img_smooth[:,-1])>2:
        raise WFSGenerationError("Light goes beyond the cell!")    
    return img_smooth


def generate_offsets(width, height, deviant = 5 ):
    
    offsets = np.zeros((width, height, 2))
    for i in range(width):
        for j in range(height):
            x = randint(-deviant,deviant)
            y = randint(-deviant,deviant)
            offsets[i,j,:]= np.asarray([x,y])
    return offsets

def draw_subs(offsets, R, border, sigma = 12):
    countw, counth = offsets.shape[:2]
    
    width = (R*2 + border) * countw
    height = (R*2 + border) * counth
    img = np.zeros((width, height))
    
    for i in range(countw):
        for j in range(counth):
            ci = i*(R*2+border) + border//2
            cj = j*(R*2+border) + border//2
            circle_pattern = drow_circle(R, offsets[i,j], sigma = sigma)
            img[ci:ci+R*2,cj:cj+R*2] = circle_pattern.copy()
            
    return img                   
            
    
if __name__ == "__main__":
    
    R = 100
    border = 10
    sigma = 10
    offsets = generate_offsets(8, 8)
    img = draw_subs(offsets, R, border, sigma = sigma)

#            
    try:
        print("kek")
        plt.figure(figsize = (10,10))
        plt.imshow(img)
    except:
        print("lol")
        pass
#        
        



import pywt
import numpy as np
from collections import Counter
import math
import matplotlib.pyplot as plt

def rotate2d(point,origin,degrees):
    """
    A rotation function that rotates a point around a point
    to rotate around the origin use [0,0]
    """
    x = point[0] - origin[0]
    yorz = point[1] - origin[1]
    newx = (x*math.cos(math.radians(degrees))) - (yorz*math.sin(math.radians(degrees)))
    newyorz = (x*math.sin(math.radians(degrees))) + (yorz*math.cos(math.radians(degrees)))
    newx += origin[0]
    newyorz += origin[1] 

    return int(newx+(0.5)*(newx/abs(newx))), int(newyorz+(0.5)*(newyorz/abs(newyorz)))

def qualitative_sub(cell, std, mean_val):
        return np.mean(cell) > std

def moving_average(a, n=3) :
    ret = np.cumsum(a, dtype=float)
    ret[n:] = ret[n:] - ret[:-n]
    return ret[n - 1:] / n

def get_interval(extremums):
    extremums = sorted(extremums)
    widths = []
    for i in range(1,len(extremums)):
        widths.append(extremums[i]-extremums[i-1])
    return widths

def get_extrememums(signal, shifting = 10):
    sort_sig =moving_average(sorted(signal), shifting)

    std = np.std(sort_sig)
    std_max = np.mean([std, np.max(sort_sig)])
    std_min = np.mean([-std, np.min(sort_sig)])
    return std_max, std_min
        
def clear_extremums(maxes):
        main_points = []
        last_point = 0
        mean_point = []
    
        for point in maxes:
            if (point-1) == last_point:
                mean_point.append(point)
            else:            
                if len(mean_point)!=0:
                    main_points.append((np.max((mean_point))))
                mean_point = []
            last_point = point
        return main_points
    
def detect_start_point( p1, p2, p3):
    diff1 = p3 - p2
    diff2 = p2 - p1
    
    if diff1 > diff2:
        return p3
    else:
        return p2
    
def get_cell_parametrs(lines):
    d1 = 0  
    d2 = 0
    d =[]
    lines = sorted(list(set(lines)))

    for i in range(2,len(lines),2):
        d.append(abs(lines[i] - lines[i-1]))
    d1 = np.median(d)
    d = []
    for i in range(1,len(lines),2):
        d.append(abs(lines[i] -lines[i-1]))
    d2 = np.median(d)

    return {"cell_width":max(d2,d1), "border":min(d2,d1)} 


def chose_optimal_interval(cd, end = 3): 
    cur_lvl = len(cd)-1
    zoom = 2
    max_interval = []
    min_interval = []
    
    coordinate = []
    while cur_lvl != end:
        std_max,  std_min= get_extrememums(cd[cur_lvl], 15)
        mask1 = list(np.where(cd[cur_lvl] > std_max)[0])
        mask2 = list(np.where(cd[cur_lvl] < std_min)[0])
        coordinate.append(mask2[0]*zoom)
        I = np.asarray(get_interval(mask1 + mask2)) #все найденные интервалы 
        
        if len(set(I))<8:
            std = np.std(I)
            max_intervals = np.where(I>std)[0]
            big_width = np.mean(I[I>std])
    
            c = []

            for i in range(1,len(max_intervals)): #нахождение минимального интервала
                c+= [sum(I[max_intervals[i-1]+1:max_intervals[i]])]
            min_width = np.median(c)
            max_interval.append(big_width*zoom)
            min_interval.append(min_width*zoom)
          #  print(big_width*zoom, min_width*zoom)
        else:
            pass
        zoom*=2
        cur_lvl-=1
   # print(np.mean(coordinate))
    return (max_interval, min_interval,np.mean(coordinate))   


def grid_cell_parametrs(img_grid):
    w = pywt.Wavelet('db2')
    max_lvl = pywt.dwt_max_level(len(img_grid), w.dec_len)-1
    coeffs = pywt.wavedec(np.mean(img_grid,axis = 0), 'db2', level = max_lvl)   
    ca, cd = coeffs[0],coeffs[1:]  
    max_interval_h, min_interval_h, coord_x = chose_optimal_interval(cd)
    
    w = pywt.Wavelet('db2')
    max_lvl = pywt.dwt_max_level(len(img_grid), w.dec_len)
    coeffs = pywt.wavedec(np.mean(img_grid,axis = 1), 'db2', level = max_lvl)   
    ca, cd = coeffs[0],coeffs[1:]  
    max_interval_w, min_interval_w, coord_y = chose_optimal_interval(cd)
    
    max_interval = max_interval_h + max_interval_w
    min_interval = min_interval_h + min_interval_w
    
    max_v = Counter(max_interval).most_common(1)[0]
    min_v = Counter(min_interval).most_common(1)[0]
    
    if min_v[1]==1:
        min_v = np.median(min_interval)
    else:
        min_v = min_v[0]
    if max_v[1]==1:
        max_v = np.median(max_interval)
    else:
        max_v = max_v[0]
    return max_v, min_v, coord_x, coord_y


def detect_grid_lines(image,  direction = 0, grid_type = 0):
    
    width, border,coord_x, coord_y = grid_cell_parametrs(image)
    
    if grid_type!=0:
        start_point_h = coord_x + width//2 
        start_point_w = coord_y + width//2 
    else:   
        h_line = np.median(image, 0)
        w_line = np.median(image, 1)

        diff_h = np.diff(moving_average(h_line, 4))
        h_maxes = np.where(abs(diff_h)>np.mean(np.abs(diff_h)))
        h_maxes = clear_extremums(h_maxes[0])

        diff_w = np.diff(moving_average(w_line, 4))
        w_maxes = np.where(abs(diff_w)>np.mean(np.abs(diff_w))) 
        w_maxes = clear_extremums(w_maxes[0])


        start_point_h = int(detect_start_point(h_maxes[1],h_maxes[2],h_maxes[3]))
        start_point_w = int(detect_start_point(w_maxes[1],w_maxes[2],w_maxes[3]))
    
   
        
#        'start_h': start_point_h, 'start_w': start_point_w, 'width_lines': w_maxes, 'height_lines': h_maxes, 
#               'cell_width':width, 'border':border
    return [start_point_h, start_point_w,  
            [], [], 
            width, border]
    
def make_gridpoints(image, cell_width, border, start_point ):
    x, y = start_point
    X = []
    Y = []
    #print(int(x), 0, -int(cell_width+border))
    
    if x!=0:
        for i in range(int(x), 0, -int(cell_width+border)):
            X.append(i)
            X.append(i+border)
        X.pop(-2)
        X = X[::-1]
    
    for i in range(int(x), image.shape[0], int(cell_width+border)):
        if i == int(x):
            continue
        X.append(i)
        X.append(i+border)

    X.pop(-1)
    X = sorted(X)
    
    if y!= 0:
        for i in range(int(y), 0, -int(cell_width+border)):
            Y.append(i)
            Y.append(i+border)
        Y.pop(-2)
        Y = Y[::-1]
    
    for i in range(int(y), image.shape[1], int(cell_width+border)):
        if i == int(y):
            continue
        Y.append(i)
        Y.append(i+border)
        
    Y.pop(-1)
    Y = sorted(Y)
    return (X,Y)

def points2grid(x_points, y_points):    
    cell = []
    cells = []
    #print(x_points)
  #  print(y_points)
    for i in range(0,len(x_points),2):
        for j in range(0,len(y_points),2):
            try:
                cell = [(x_points[i], x_points[i+1],x_points[i],x_points[i+1]),
                        (y_points[j],y_points[j], y_points[j+1],y_points[j+1])]
                cells.append(cell)
            except:
                continue
    
    return cells    

def rotate(cells, origin, degrees):
    for i in range(len(cells)):
        cell = list(zip(*cells[i]))
        X = []
        Y = []
        for j in range(len(cell)):
            x,y = rotate2d(cell[j],origin,degrees)
            X.append(x)
            Y.append(y)
        cells[i] = [tuple(X), tuple(Y)]
    return cells


# -*- coding: utf-8 -*-
"""
Created on Fri Jun 28 13:48:30 2019

@author: Zoya
"""

import pickle 
import numpy as np 
from ._wfs import make_gridpoints, points2grid, detect_grid_lines
import matplotlib.pyplot as plt
import matplotlib.lines as lines
from skimage.transform import rotate
from typing import Union


class Geometry():   
    
    only_wavelets = 0
    def __init__(self, image: np.ndarray, make_func = 'not_auto', border = 0, 
                         cell_width = 0, start_point = [0,0], angle = 0):
        self.image = image
        self._border = border
        self._cell_width = cell_width
        self._start_point = start_point
        self._cells = []
        self._rotate = angle
        self.mask = []
        #self.only_wavelets = only_wavelets
        if make_func.lower() == 'auto':
            self.auto_make()
        elif make_func.lower() == 'not_auto':
            if border!= 0 and cell_width!=0:
                self.__make()
            pass      
        
    
    def __make(self) -> None:
        self.image = rotate(self.image, -self._rotate, resize=False, center=None, order=1,
                 mode='constant', cval=0, clip=True, preserve_range=False)
        
        if isinstance(self._start_point[0],type(np.nan)) or self._start_point[0]<0:
            self._start_point[0] = 0
            
        if isinstance(self._start_point[1],type(np.nan)) or self._start_point[0]<0:
            self._start_point[1] = 0
        
        #print(self._cell_width, self._border, self._start_point )
        points = make_gridpoints(self.image, self._cell_width, self._border, self._start_point )
        self._cells = points2grid(*points)
        #print(self._cells)
        #center = self.image.shape[0]//2, self.image.shape[1]//2
        #self._cells = rotate(self._cells, center, self._rotate)
        self.mask = list(range(len(self._cells)))
        return 
    
    def auto_make(self, diraction = 0) -> dict:
        options = detect_grid_lines(self.image,  direction = 0, grid_type = self.only_wavelets)
        self._border = options[-1]
        self._cell_width = options[-2]
        self._start_point = options[:2]
        
        self.__make()
        
        return {'border': self._border, 'cell_width': self._cell_width, 'start_point': self._start_point}
    
    def set_image(self, image: np.ndarray) -> None:
        self.image = image
        
    def set_options(self, border = None, cell_width = None, 
                      shift = None, swap = False, rotate = 0) -> None:
        if border is not None:
            self._border = border
        if cell_width is not None:
            self._cell_width = cell_width
        if shift is not None:
            self._start_point[0] += shift[0]
            self._start_point[1] += shift[1]
        if swap:
            self._border,  self._cell_width  =  self._cell_width, self._border 
        self._rotate = rotate
        self.__make()
        
    @property
    def geometry(self) -> np.ndarray:
        return np.asarray(self._cells)[self.mask]
        
    @property
    def options(self) -> list:
        return  {'border': self._border, 'cell_width': self._cell_width, 'start_point': self._start_point}
    
    def show(self):        
        plt.figure(figsize = (8,8))         
        plt.imshow(self.image)
        for i in range(len(self._cells)):
            x0, x1, x2, x3 = self._cells[i][1]
            y0, y1, y2, y3 = self._cells[i][0]
            
            plt.plot([x0, x1], [y0, y1], 
                     [x0, x2], [y0, y2],
                     [x2, x3], [y2, y3],
                     [x3, x1], [y3, y1],color = 'r')
      
        plt.show()

   
        


    
    

    
    
    
        



import h5py  # type: ignore
import numpy  # type: ignore
from pathlib import Path
from warnings import warn
from ._wfs import qualitative_sub
from .shift_detector_numba import translations
from skimage.feature import register_translation
from skimage.transform import rotate
from typing import Union
from .geometry import Geometry
from skimage.transform import rotate

import pickle
import matplotlib.pyplot as plt
DataSources = Union[str, h5py.File, numpy.ndarray]


    

class WFSError(Exception):
    pass

class Frame():
    def __init__(self, image, geometry, reference, mask = None, qualitative_function = None):
        self._geometry = geometry
        self.image = image
        self.reference = reference
        self.qualitative_function = qualitative_function if qualitative_function else qualitative_sub
        self.mask = mask

        
    def __getitem__(self, sub_number: int) -> numpy.ndarray: 
        if not isinstance(self.mask, type(None)):
            cell = self._geometry.geometry[self.mask][sub_number]
        else:
             cell = self._geometry.geometry[sub_number]
        sx, ssx , sy, ssy = list(map(int,[cell[0][0], cell[0][1],
                                       cell[1][0], cell[1][2]]))           
        return self.image[sx:ssx,sy:ssy]
    
    def __len__(self):
        if not isinstance(self.mask, type(None)):
            return len(self._geometry.geometry[self.mask])
        else:
            return len(self._geometry.geometry)
    
    def cell_quality(self, i:int)->bool:
        cell = self._geometry.geometry[i]
        sx, ssx , sy, ssy = list(map(int,[cell[0][0], cell[0][1],
                                       cell[1][0], cell[1][2]]))
        return self.qualitative_function(self.image[sx:ssx,sy:ssy], 
                               numpy.std(self.image),
                               numpy.mean(self.image))
        
    
    def get_offset(self, sub_number):        
        return register_translation(self.__getitem__(self.reference),
                             self.__getitem__(sub_number))[0]
        
    def set_image(self, image):
        self.image = rotate(image, -self._geometry._rotate, resize=False, center=None, order=1,
                 mode='constant', cval=0, clip=True, preserve_range=False)
    
    def offsets(self, dformat = 'row'):
        offsets = []
        
        for i in range(len(self._geometry.geometry)):
             ofst = list(self.get_offset(i))
             offsets.append([-ofst[0], -ofst[1]])
        offsets = numpy.asarray(offsets)
        if dformat == 'row':
            return offsets
        elif dformat == 'imat':
            return numpy.append(offsets[:,0],offsets[:,1])
    
    def __get_all_subaperturs(self):
        subs = []
        for i in range(len(self._geometry.geometry)):
             subs.append(self[i])
        ref_sub = self[self.reference]
        return numpy.asarray(subs), ref_sub
    
    def qoffsets(self):
        subs, ref_sub = self.__get_all_subaperturs()
        offsets = translations(len(subs), numpy.asarray(subs), ref_sub)
        return numpy.asarray(sorted(offsets, key = lambda a: a[0]))[:,1:]
    

class WFSData():

    def __init__(self, source: DataSources, dataset_name = "data", qualitative_function = None) -> None:
        
        self.dataset_name = dataset_name
        self._reference = 81
        self._source = None
        self.geometry = None
        self.h5f_stream = None        
        self._qualitative_function = qualitative_function if qualitative_function else qualitative_sub
        
        self.__load_source(source)
        self.__load_geometry()
        
        
        self._mask = False
        self._frame = Frame(self._source[0], self.geometry, self._reference,
                                        qualitative_function = self._qualitative_function)
        self.quality_mask = []
        

    def __load_source(self, source) -> None:
       
        if isinstance(source, str):            
            if not Path(source).exists():
                raise WFSError("NameError: not file or directory with name %s."%source)
            if "h5" not in source:
                raise WFSError("TypeError: file is not a hdf5-file format type."%source)
            self.h5f_stream = h5py.File(source,'r')
            
            if self.dataset_name in  self.h5f_stream.keys():
                if len( self.h5f_stream[self.dataset_name].shape)>2:
                    self._source = self.h5f_stream[self.dataset_name]
                else:
                    self._source = numpy.expand_dims(self.h5f_stream[self.dataset_name], axis=0)
            elif "/" in source:                
                print("!!!")
                if len( self.h5f_stream[self.dataset_name].shape)>2:
                    self._source = self.h5f_stream[self.dataset_name]
                else:
                    self._source = numpy.expand_dims(self.h5f_stream[self.dataset_name], axis=0)                

            if ("cell_width" in self.h5f_stream.keys() and 
                            "border" in self.h5f_stream.keys() and 
                            "start_point" in self.h5f_stream.keys()) :
                cell_width = self.h5f_stream["cell_width"][0]
                border = self.h5f_stream["border"][0]
                start_point = self.h5f_stream["start_point"][0]
                self.geometry = Geometry(image = self._source[0], 
                                         cell_width = cell_width, 
                                         border = border,
                                         start_point = start_point)
            
          #  h5f.close() 
        elif isinstance(source, h5py.File):
            if self.dataset_name in source.keys():
                self._source = source[self.dataset_name]
            if  ("cell_width" in source.keys() and 
                            "border" in source.keys() and 
                            "start_point" in source.keys()):
                cell_width = source["cell_width"][0]
                border = source["border"][0]
                start_point = source["start_point"][0]
                self.geometry = Geometry(image = self._source[0], 
                                         cell_width = cell_width, 
                                         border = border,
                                         start_point = start_point) 
            if self.dataset_name not in source.keys():
                raise WFSError('Сan not read the file. Be sure to have the keys "date" for data WFS.')                
        elif isinstance(source, numpy.ndarray):
            self._source = source
            if len(source.shape) == 2:
                self._source =  self._source.reshape(1,self._source.shape[0], self._source.shape[1])
        else:
            raise WFSError("I am so sorry! I cant't open file with this format.")      
    
    def close_stream(self):
        self.h5f_stream.close()
        
    def __load_geometry(self) -> None:
        if not isinstance(self.geometry, type(None)):
            return
        self.geometry = Geometry(self._source[0], 'auto')
        warn("WARNING: Set the geometry for the file!", UserWarning)
   
    def __iter__(self):
        raise NotImplementedError()  

    def __getitem__(self, frame_number: int) -> dict: 
#         if self._mask == True:
#             self._frame.mask = self.quality_mask
#         else:
#             self._frame.mask = None
        self._frame.set_image(self._source[frame_number])
        return self._frame
    
    def save(self, name=None) -> None:
        data_name = name if name is not None else self.dataset_name
        if "h5" not in data_name:
            warn("File name is not hdf5 file format", UserWarning)        
        with h5py.File(data_name, 'a') as f:           
            #f.create_dataset(self.dataset_name, data=self._source)
            f.create_dataset("cell_width", data= numpy.asarray([self.geometry._cell_width]))
            f.create_dataset("border", data=numpy.asarray([self.geometry._border]))
            f.create_dataset("start_point", data=numpy.asarray([self.geometry._start_point]))
            
    @property
    def qualitative_function(self):
        return self._qualitative_function 
    
    @qualitative_function.setter
    def qualitative_function(self, qualitative_function):
        self._qualitative_function = qualitative_function
        self._frame.qualitative_function = qualitative_function
        self.quality_mask = self.__get_quality_mask()
    
    @property
    def reference(self) -> Union[int, None]:
        return self._reference

    @reference.setter
    def reference(self, ref_num):
        self._reference = ref_num
        self._frame.reference = self._reference
        
    @property
    def good_only(self):
        return self._mask
    
    @good_only.setter
    def good_only(self, state) -> bool:
        self._mask = state
        if self._mask:
            self.quality_mask = self.__get_quality_mask()
        self.geometry.mask = self.quality_mask
        return 
    
    def __get_quality_mask(self):
        geometry = self.geometry.geometry
        mask = numpy.zeros(geometry.shape[0])
        mask = mask.astype(bool)
        for i in range(len(geometry)):
             if self._frame.cell_quality(i):
                 mask[i] = True 
             else:
                 mask[i] = False
        return mask

    def show_geometry(self, show_type = "numered") -> None:
        self._frame.set_image(self._source[0])
        plt.figure(figsize = (10,10))   
        img_to_show = rotate(self._source[0], -self.geometry._rotate, resize=False, center=None, order=1,
                 mode='constant', cval=0, clip=True, preserve_range=False) 
        plt.imshow(img_to_show)
        plt.title(show_type+" image")
        sx = 1
        sy = 11
       # if self._mask:
          #  geometry = self.geometry.geometry[self.quality_mask]
      #  else:
        geometry = self.geometry.geometry
        for i in range(len(geometry)):

            weight = 'normal'
            fontsize = 12
            if self._mask or self._frame.cell_quality(i):
                color = '#f6416e'
            else:
                color = 'c'
            if i == self._reference:
                color = 'r'
                weight = 'bold'
            x0, x1, x2, x3 = geometry[i][1]
            y0, y1, y2, y3 = geometry[i][0]
            
            text = ""
            if show_type == "numered":
                text = "%d"%i
            elif show_type == "offsets":
                ofst = self._frame.get_offset(i)
                text = "%.1f,  %.1f"% (-ofst[0], -ofst[1])
                fontsize = 10
                sx = 2
                sy = 12
                
            plt.text(x0+sx, y0+sy, text, color = color, fontsize = fontsize, weight=weight)
            plt.plot([x0, x1], [y0, y1], 
                     [x0, x2], [y0, y2],
                     [x2, x3], [y2, y3],
                     [x3, x1], [y3, y1],color = color)
      
        plt.show()
        
   



# -*- coding: utf-8 -*-
"""
Created on Wed Apr 15 11:22:22 2020

@author: mi
"""

from distutils.core import setup,Extension
from Cython.Build import cythonize
import numpy


setup(
    ext_modules=cythonize("shift_detector.pyx"),
    include_dirs=[numpy.get_include()]
)    

setup(
    ext_modules=[
        Extension("shift_detector", ["shift_detector.c"],
                  include_dirs=[numpy.get_include()]),
    ],
)

# Or, if you use cythonize() to make the ext_modules list,
# include_dirs can be passed to setup()




# -*- coding: utf-8 -*-
"""
Created on Wed Apr 15 11:00:47 2020

@author: mi
"""

from cython.parallel import prange
import numpy as np
cimport numpy as np
#cimport wfs.Frame as Frame

ctypedef np.float_t DTYPE_t

cpdef tuple[int] translation(DTYPE_t[:]  ref1, DTYPE_t[:]  ref2):
    s = np.abs(np.fft.ifft2(np.fft.fft2(ref1) * np.conjugate(np.fft.fft2(ref2))))
    x, y = np.where(s == np.min(s))
    return x, y

cpdef list translations(DTYPE_t[:] subs, DTYPE_t[:] ref):
    cdef list offsets = []
    cdef list ofst = [2,2]
    cdef int i
    cdef int maxi = len(subs)
    cdef x, y
    x = 0    
    y = 0
    cdef DTYPE_t s
    for i in prange(maxi, nogil=True):
           # ofst = translation(subs[i], ref)
            s = np.abs(np.fft.ifft2(np.fft.fft2(subs[i]) * np.conjugate(np.fft.fft2(ref))))
            x, y = np.where(s == np.min(s))
            offsets.append(ofst)
    return offsets


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['wfslib']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['wfslib = entry:main']}

setup_kwargs = {
    'name': 'wfslib',
    'version': '0.1.200',
    'description': 'Programming module for wave front data processing',
    'long_description': "**wfslib** is open library for wave front data processing: \n\n* different data type usage (hfd, bim, numpy);\n* automatic geometry calculation;\n* offsets calculation;\n* settings-tools for manage yours data.\n\n### Example of wfslib work\n\n\n```python\nfrom wfslib.wfs import WFSData\n\nwfs = WFSData('../data/subpixel_test.h5', dataset_name = 'wfss/n0/detector') #Load data\n\np = wfs.geometry.options #Geometry options\nprint(p)\nwfs.reference = 8\n\n#Change geometry options\nwfs.geometry.set_options(shift=(-p['start_point'][0]+1,-p['start_point'][1]+1), border = 0, cell_width = p['cell_width']-1)\n#Visualization\nwfs.show_geometry()\n```\n\n\n\n    {'border': 4.0, 'cell_width': 110.0, 'start_point': [168, 131]}\n    \n\n\n![jpg](https://sun4-12.userapi.com/G3ulaQxVXOs1Wubv6BmAOLaFN2l-v0IVHuaAaw/Xu-2xvIdMVY.jpg)\n\n\n\n```python\nwfs[0].offsets()[:5] #Calculate offsets\n```\n\n\n\n\n\n    array([[-0., -0.],\n           [-0., -0.],\n           [-1., -1.],\n           [-1.,  1.],\n           [22.,  9.]])\n\n\n\n\n",
    'author': 'Zoya',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
